/*
 * Segment Datenübernahme und Aufbereitung (DUA), Fehleranalyse TLS spontan
 * Copyright (C) 2016 by Regierungspräsidium Tübingen, Landesstelle für Straßentechnik
 * Copyright 2016 by Kappich Systemberatung Aachen
 * Copyright (C) 2007 BitCtrl Systems GmbH 
 * Basiert auf der gleichnamigen Klasse aus de.bsvrz.dua.fehlertls.tls, BitCtrl Systems GmbH
 * 
 * SWE de.bsvrz.dua.fehlertlsspontan (Fehleranalyse TLS spontan)
 * 
 * This program is free software; you can redistribute it and/or modify it under the terms of the 
 * GNU General Public License as published by the Free Software Foundation; either version 3 of 
 * the License, or (at your option) any later version.
 * This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; 
 * without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. 
 * See the GNU General Public License for more details.
 * You should have received a copy of the GNU General Public License along with this program; 
 * if not, see <http://www.gnu.org/licenses/>.
 *
 * Dieses Programm ist freie Software. Sie können es unter den Bedingungen der GNU General Public 
 * License, wie von der Free Software Foundation veröffentlicht, weitergeben und/oder modifizieren, 
 * entweder gemäß Version 3 der Lizenz oder (nach Ihrer Option) jeder späteren Version.
 * Die Veröffentlichung dieses Programms erfolgt in der Hoffnung, dass es Ihnen von Nutzen sein wird, 
 * aber OHNE IRGENDEINE GARANTIE, sogar ohne die implizite Garantie der MARKTREIFE oder der 
 * VERWENDBARKEIT FÜR EINEN BESTIMMTEN ZWECK. Details finden Sie in der GNU General Public License.
 * Sie sollten ein Exemplar der GNU General Public License zusammen mit diesem Programm erhalten haben. 
 * Falls nicht, siehe <http://www.gnu.org/licenses/>.
 * Eine deutsche Übersetzung findet sich unter http://www.gnu.de/documents/gpl.de.html.
 *
 * Contact Information/Kontakt:
 * Regierungspräsidium Tübingen
 * Landesstelle für Straßentechnik
 * Heilbronner Straße 300 - 302
 * 70469 Stuttgart
 * Phone: +49 0711 8910-200
 * mailto: abteilung9@rpt.bwl.de
 */

package de.bsvrz.dua.fehlertlsspontan.tls;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.config.SystemObject;
//import de.bsvrz.dua.fehlertls.de.DeFaException;
import de.bsvrz.dua.fehlertlsspontan.enums.TlsFehlerAnalyse;
import de.bsvrz.sys.funclib.bitctrl.dua.DUAKonstanten;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.sys.funclib.operatingMessage.MessageGrade;
import de.bsvrz.sys.funclib.operatingMessage.MessageTemplate;
import de.bsvrz.sys.funclib.operatingMessage.MessageType;
import de.bsvrz.sys.funclib.operatingMessage.OperatingMessage;

/**
 * TLS-Hierarchieelement EAK.
 * 
 * @author Regierungspräsidium Tübingen, Landesstelle für Straßentechnik, Thomas Bräuner
 */
public class Eak extends TlsGeraet {
	
	private static final MessageTemplate TEMPLATE = new MessageTemplate(
			MessageGrade.ERROR,
			MessageType.APPLICATION_DOMAIN,
			MessageTemplate.fixed("Alle DE des EAK "),
			MessageTemplate.object(),
			MessageTemplate.fixed(" am Steuermodul "),
			MessageTemplate.variable("sm"),
			MessageTemplate.fixed(" defekt. "),
			MessageTemplate.fixed("DE und EAK "),
			MessageTemplate.object(),
			MessageTemplate.fixed(" am Steuermodul "),
			MessageTemplate.variable("sm"),
			MessageTemplate.fixed(" instand setzen. "),
			MessageTemplate.ids()
	).withIdFactory(message -> message.getObject().getPidOrId() + " [DUA-FS-FU]");
	
	/**
	 * Standardkonstruktor.
	 * 
	 * @param dav
	 *            Datenverteiler-Verbindung
	 * @param objekt
	 *            ein Systemobjekt vom Typ <code>typ.eak</code>
	 * @param elter
	 *            das in der TLS-Hierarchie über diesem Gerät liegende Gerät
	 */
	protected Eak(ClientDavInterface dav, SystemObject objekt, TlsGeraet elter) {
		super(dav, objekt, elter);
		for (SystemObject deObj : this.objekt.getNonMutableSet("De").getElements()) {
			if (deObj.isValid()) {
//				try {
					Data deKonfig = deObj.getConfigurationData(dav.getDataModel().getAttributeGroup("atg.de"));
					if(deKonfig != null) {
						if(deKonfig.getUnscaledValue("Cluster").intValue() == DUAKonstanten.NEIN){
							De de = new De(dav, deObj, this);
							this.kinder.add(de);
						} else {
							Debug.getLogger().info("DE " + deObj
									+ " ist als Sammelkanal konfiguriert und wird daher ignoriert.");							
						}
					} else {
						Debug.getLogger().warning("DE " + deObj
								+ " besitzt keine Konfigurationsdaten (innerhalb von ATG \"atg.de\") und wird ignoriert.");
					}
			}
		}
	}

	/**
	 * {@inheritDoc}
	 * <br>In diesem Fall ein EAK.
	 */
	@Override
	public Art getGeraeteArt() {
		return Art.EAK;
	}

	/**
	 * {@inheritDoc}
	 * <br>In diesem Fall alle DE eines EAK defekt.
	 */
	@Override
	public void publiziereFehler(long zeitStempel) {
		OperatingMessage message = TEMPLATE.newMessage(objekt);
		message.addId("[DUA-FS-FU05]");
		message.put("sm", getVater().getObjekt());
		if (darfBetriebsmeldungSenden())
			this.publiziere(message);

		for (De de : getDes()) {
			if (de.getObjekt().isOfType("typ.deWzg") || de.getObjekt().isOfType("typ.deVlt")) {
				de.publiziereFehlerUrsache(zeitStempel,	TlsFehlerAnalyse.EAK_ALLE_DE_DEFEKT);
			}
		}
	}
	
	/**
	 * Bestimmt, ob ein DE eine Betriebsmeldung absetzen darf
	 * oder ob das der EAK macht, weil alle DE auf Störung sind.
	 * 
	 * @param zeitStempel
	 *            der Zeitstempel des Fehlers
	 *            
	 * @return true, wenn alle DE gestört sind, sonst false
	 */
	public boolean alleDeGestoert(long zeitStempel) {
		boolean erg = true;
		for (De de : getDes()) {
			if (!de.isGestoert() && de.isInitialisiert()) {
				erg = false;
			}
		}
		if (erg) {
			if (!((Sm)elter).alleDeGestoert(zeitStempel)) {
				// Es sind nicht alle DE am Eltern-SM gestört, sonst muss der SM den Fehler publizieren
				publiziereFehler(zeitStempel);
			}
		}
		return erg;
	}

}
